@@ -482,6 +482,23 @@ PL_EXPORT(IOENC)	PL_atom_to_encoding(atom_t name);
 PL_EXPORT(atom_t)	PL_encoding_to_atom(IOENC enc);
 #endif
 
+PL_EXPORT(char *)	PL_qlf_getString(IOSTREAM *, size_t *len);
+PL_EXPORT(int64_t)	PL_qlf_getInt64(IOSTREAM *);
+PL_EXPORT(int)		PL_qlf_getInt32(IOSTREAM *s);
+PL_EXPORT(unsigned int)	PL_qlf_getUInt(IOSTREAM *fd);
+PL_EXPORT(int)		PL_qlf_getInt(IOSTREAM *);
+PL_EXPORT(double)	PL_qlf_getFloat(IOSTREAM *);
+// TODO: PL_EXPORT(pl_wchar_t *) PL_qlf_getStringW(IOSTREAM *, size_t *len);
+
+PL_EXPORT(void)		PL_qlf_putString(const char *, size_t len, IOSTREAM *);
+PL_EXPORT(void)		PL_qlf_putInt64(int64_t, IOSTREAM *);
+PL_EXPORT(void)		PL_qlf_putInt32(int32_t, IOSTREAM *);
+PL_EXPORT(void)		PL_qlf_putUInt(unsigned int i, IOSTREAM *fd);
+PL_EXPORT(void)		PL_qlf_putInt(int v, IOSTREAM *fd);
+PL_EXPORT(void)		PL_qlf_put_Float(double, IOSTREAM *);
+// TODO: PL_EXPORT(void) PL_qlf_putStringW(const pl_wchar_t *s, size_t len, IOSTREAM *fd);
+
+
 #ifdef __cplusplus
 }
 #endif
