@@ -292,6 +292,12 @@ set_modified(DECL_LD gen_t gen)
   }
 }
 
+static void
+retract_clause_gen(Clause clause, gen_t generation)
+{ if ( clause->generation.erased > generation )
+    clause->generation.erased = generation;
+}
+
 #define transaction_commit(_) LDFUNC(transaction_commit, _)
 static int
 transaction_commit(DECL_LD)
@@ -322,19 +328,27 @@ transaction_commit(DECL_LD)
 	  cl->generation.created = GEN_TR_ASSERT_ERASE;
 	}
       } else if ( lgen == GEN_NESTED_RETRACT )
-      { retract_clause(cl, gen_commit);
+      { retract_clause_gen(cl, gen_commit);
       } else
       { DEBUG(MSG_COMMIT,
 	      Sdprintf("Commit erased clause %p for %s\n",
 		       cl, predicateName(cl->predicate)));
 	ATOMIC_DEC(&cl->tr_erased_no);
-	retract_clause(cl, gen_commit);
+	retract_clause_gen(cl, gen_commit);
       }
     }
     MEMORY_RELEASE();
     GD->_generation = gen_commit;
     PL_UNLOCK(L_GENERATION);
 
+    FOR_TABLE(LD->transaction.clauses, n, v)
+    { Clause cl = n;
+      uintptr_t lgen = (uintptr_t)v;
+
+      if ( !IS_ASSERT_GEN(lgen) )
+	retract_clause(cl, cl->generation.erased);
+    }
+
     set_modified(gen_commit);
     destroyHTable(LD->transaction.clauses);
     LD->transaction.clauses = NULL;
@@ -603,7 +617,7 @@ transaction(DECL_LD term_t goal, term_t constraint, term_t lock, int flags)
 #define TR_UNLOCK() PL_mutex_unlock(mutex)
 #else
     int tid = 1;			/* without threads we get -2 */
-#define TR_LOCK() (void)0
+#define TR_LOCK() TRUE
 #define TR_UNLOCK() (void)0
 #endif
 
@@ -614,10 +628,13 @@ transaction(DECL_LD term_t goal, term_t constraint, term_t lock, int flags)
     LD->transaction.id         = goal;
     rc = callProlog(NULL, goal, PL_Q_PASS_EXCEPTION, NULL);
     if ( rc && (flags&TR_TRANSACTION) )
-    { if ( constraint )
-      { TR_LOCK();
-	LD->transaction.gen_start = global_generation();
-	rc = callProlog(NULL, constraint, PL_Q_PASS_EXCEPTION, NULL);
+    { int locked = FALSE;
+      if ( constraint )
+      { if ( (rc=TR_LOCK()) )
+	{ locked = TRUE;
+	  LD->transaction.gen_start = global_generation();
+	  rc = callProlog(NULL, constraint, PL_Q_PASS_EXCEPTION, NULL);
+	}
       }
       if ( rc && (flags&TR_BULK) )
       { transaction_updates(&updates);
@@ -626,9 +643,9 @@ transaction(DECL_LD term_t goal, term_t constraint, term_t lock, int flags)
       if ( rc )
       { rc = ( transaction_commit_tables() &&
 	       transaction_commit() );
-	if ( constraint ) TR_UNLOCK();
+	if ( locked ) TR_UNLOCK();
       } else
-      { if ( constraint ) TR_UNLOCK();
+      { if ( locked ) TR_UNLOCK();
 	LD->transaction.generation = 0;	/* avoid recording the rollback */
 	transaction_discard();
 	transaction_rollback_tables();
@@ -837,4 +854,3 @@ BeginPredDefs(transaction)
   PRED_DEF("pred_generations",    1, pred_generations,    META)
 #endif
 EndPredDefs
-
