@@ -3,8 +3,8 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1985-2022, University of Amsterdam
-                              VU University Amsterdam
+    Copyright (c)  1985-2023, University of Amsterdam
+			      VU University Amsterdam
 			      CWI, Amsterdam
 			      SWI-Prolog Solutions b.v.
     All rights reserved.
@@ -72,12 +72,6 @@
 
 #define	     BFR (LD->choicepoints)	/* choicepoint registration */
 
-#if sun
-#include <prof.h>			/* in-function profiling */
-#else
-#define MARK(label)
-#endif
-
 #define newChoice(type, fr) LDFUNC(newChoice, type, fr)
 static Choice	newChoice(DECL_LD choice_type type, LocalFrame fr);
 
@@ -124,115 +118,6 @@ typedef foreign_t (*NdetFunc10)(term_t a1, term_t a2, term_t a3, term_t a4,
 				term_t a9, term_t a10, control_t);
 
 
-#if COUNTING
-
-/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-The counting code has been added   while investigating the time critical
-WAM  instructions.  The  current  implementation  runs  on  top  of  the
-information  provided  by  code_info   (from    pl-comp.c)   and  should
-automatically addapt to modifications in the VM instruction set.
-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
-
-typedef struct
-{ code  code;
-  int	times;
-  int  *vartimesptr;
-} count_info;
-
-#define MAXVAR 8
-
-static count_info counting[I_HIGHEST];
-
-static void
-count(code c, Code PC)
-{ const code_info *info = &codeTable[c];
-
-  counting[c].times++;
-  switch(info->argtype)
-  { case CA1_VAR:
-    case CA1_FVAR:
-    case CA1_CHP:
-    { int v = (int)*PC;
-
-      v -= ARGOFFSET/sizeof(word);
-      assert(v>=0);
-      if ( v >= MAXVAR )
-	v = MAXVAR-1;
-
-      if ( !counting[c].vartimesptr )
-      { int bytes = sizeof(int)*MAXVAR;
-
-	counting[c].vartimesptr = allocHeapOrHalt(bytes);
-	memset(counting[c].vartimesptr, 0, bytes);
-      }
-      counting[c].vartimesptr[v]++;
-    }
-  }
-}
-
-
-static void
-countHeader()
-{ int m;
-  int amax = MAXVAR;
-  char last[20];
-
-  Sfprintf(Scurout, "%-13s %8s ", "Instruction", "times");
-  for(m=0; m < amax-1; m++)
-    Sfprintf(Scurout, " %8d", m);
-  Ssprintf(last, ">%d", m);
-  Sfprintf(Scurout, " %8s\n", last);
-  for(m=0; m<(31+amax*8); m++)
-    Sputc('=', Scurout);
-  Sfprintf(Scurout, "\n");
-}
-
-
-static int
-cmpcounts(const void *p1, const void *p2)
-{ const count_info *c1 = p1;
-  const count_info *c2 = p2;
-
-  return c2->times - c1->times;
-}
-
-
-word
-pl_count()
-{ int i;
-  count_info counts[I_HIGHEST];
-  count_info *c;
-
-  countHeader();
-
-  memcpy(counts, counting, sizeof(counts));
-  for(i=0, c=counts; i<I_HIGHEST; i++, c++)
-    c->code = i;
-  qsort(counts, I_HIGHEST, sizeof(count_info), cmpcounts);
-
-  for(c = counts, i=0; i<I_HIGHEST; i++, c++)
-  { const code_info *info = &codeTable[c->code];
-
-    Sfprintf(Scurout, "%-13s %8d ", info->name, c->times);
-    if ( c->vartimesptr )
-    { int n, m=MAXVAR;
-
-      while(m>0 && c->vartimesptr[m-1] == 0 )
-	m--;
-      for(n=0; n<m; n++)
-	Sfprintf(Scurout, " %8d", c->vartimesptr[n]);
-    }
-    Sfprintf(Scurout, "\n");
-  }
-
-  succeed;
-}
-
-#else /* ~COUNTING */
-
-#define count(id, pc)			/* no debugging not counting */
-
-#endif /* COUNTING */
 
 		 /*******************************
 		 *	     DEBUGGING		*
@@ -240,7 +125,7 @@ pl_count()
 
 #if defined(O_DEBUG) || defined(SECURE_GC) || defined(O_MAINTENANCE)
 #define loffset(p) LDFUNC(loffset, p)
-static intptr_t
+static size_t
 loffset(DECL_LD void *p)
 { if ( p == NULL )
     return 0;
@@ -260,7 +145,7 @@ DbgPrintInstruction(LocalFrame FR, Code PC)
   { GET_LD
 
     if ( ofr != FR )
-    { Sfprintf(Serror, "#%ld at [%ld] predicate %s\n",
+    { Sfprintf(Serror, "#%zd at [%ld] predicate %s\n",
 	       loffset(FR),
 	       levelFrame(FR),
 	       predicateName(FR->predicate));
@@ -306,7 +191,7 @@ in several virtual machine instructions.  Currently covers:
 
 	* Pending signals
 	* pthread_cancel() requested
-        * Activation of the profiler
+	* Activation of the profiler
 	* out-of-stack signalled
 	* active depth-limit
 	* attributed variable wakeup
@@ -318,7 +203,7 @@ updateAlerted(PL_local_data_t *ld)
 { int mask = 0;
 
   WITH_LD(ld)
-  { if ( is_signalled() )		        mask |= ALERT_SIGNAL;
+  { if ( is_signalled() )			mask |= ALERT_SIGNAL;
     if ( !truePrologFlag(PLFLAG_VMI_BUILTIN) ||
 	 ld->prolog_flag.occurs_check != OCCURS_CHECK_FALSE )
       ld->slow_unify = TRUE;  /* see VMI B_UNIFY_VAR */
@@ -650,37 +535,37 @@ ssu_or_det_failed(DECL_LD LocalFrame fr)
   { switch(argc) \
     { case 0: \
 	(*(NdetFunc0)f)(c); \
-        break; \
+	break; \
       case 1: \
 	(*(NdetFunc1)f)(0,(c)); \
 	break; \
       case 2: \
 	(*(NdetFunc2)f)(0,0,(c)); \
-        break; \
+	break; \
       case 3: \
 	(*(NdetFunc3)f)(0,0,0,(c)); \
-        break; \
+	break; \
       case 4: \
 	(*(NdetFunc4)f)(0,0,0,0,(c)); \
-        break; \
+	break; \
       case 5: \
 	(*(NdetFunc5)f)(0,0,0,0,0,(c)); \
-        break; \
+	break; \
       case 6: \
 	(*(NdetFunc6)f)(0,0,0,0,0,0,(c)); \
-        break; \
+	break; \
       case 7: \
 	(*(NdetFunc7)f)(0,0,0,0,0,0,0,(c)); \
-        break; \
+	break; \
       case 8: \
 	(*(NdetFunc8)f)(0,0,0,0,0,0,0,0,(c)); \
-        break; \
+	break; \
       case 9: \
 	(*(NdetFunc9)f)(0,0,0,0,0,0,0,0,0,(c)); \
-        break; \
+	break; \
       case 10: \
 	(*(NdetFunc10)f)(0,0,0,0,0,0,0,0,0,0,(c)); \
-        break; \
+	break; \
       default: \
 	assert(0); \
     } \
@@ -860,47 +745,74 @@ call_term(DECL_LD Module mdef, term_t goal)
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 frameFinished() is used for two reasons:   providing hooks for the (GUI)
 debugger  for  updating   the   stack-view    and   for   dealing   with
-call_cleanup/3.  Both may call-back the Prolog engine.
+call_cleanup/2 family.  Both may call-back the Prolog engine.
+
+The helper callCleanupHandler() deals with the cleanup family. Currently
+the predicate is always
+
+    setup_call_catcher_cleanup(Setup, Goal, Catcher, Cleanup).
 
 Note that the cleanup handler is called while protected against signals.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-#define callCleanupHandler(fr, reason) LDFUNC(callCleanupHandler, fr, reason)
+#define callCleanupHandler(fr, reason) \
+	LDFUNC(callCleanupHandler, fr, reason)
+
 static void
 callCleanupHandler(DECL_LD LocalFrame fr, enum finished reason)
 { if ( false(fr, FR_CATCHED) )		/* from handler */
   { size_t fref = consTermRef(fr);
     fid_t cid;
-    term_t catcher;
-
-    assert(fr->predicate == PROCEDURE_setup_call_catcher_cleanup4->definition);
+    size_t arg_catcher = 0;
+    size_t arg_cleanup = 0;
+    term_t clean;
+    wakeup_state wstate;
 
     if ( !(cid=PL_open_foreign_frame()) )
       return;				/* exception is in the environment */
 
     fr = (LocalFrame)valTermRef(fref);
-    catcher = consTermRef(argFrameP(fr, 2));
-
-    set(fr, FR_CATCHED);
-    if ( unify_finished(catcher, reason) )
-    { term_t clean;
-      wakeup_state wstate;
-
-      fr = (LocalFrame)valTermRef(fref);
-      clean = consTermRef(argFrameP(fr, 3));
-      if ( saveWakeup(&wstate, FALSE) )
-      { int rval;
-
-	startCritical();
-        rval = call_term(contextModule(fr), clean);
-	if ( !endCritical() )
-	  rval = FALSE;
-	if ( !rval && exception_term )
-	  wstate.flags |= WAKEUP_KEEP_URGENT_EXCEPTION;
-	restoreWakeup(&wstate);
-      }
+    switch(fr->predicate->functor->arity)
+    { case 2:		/* call_cleanup(Goal, Cleanup) */
+	arg_cleanup = 2;
+        break;
+      case 3:		/* setup_call_cleanup(Setup, Goal, Cleanup) */
+	arg_cleanup = 3;
+        break;
+      case 4:		/* setup_call_catcher_cleanup(Stp, Goal, Catcher, Cln) */
+	arg_cleanup = 4;
+        arg_catcher = 3;
+	break;
+      default:
+	assert(0);
     }
 
+    set(fr, FR_CATCHED);
+
+			/* Unify the catcher */
+    if ( arg_catcher )
+    { term_t catcher = consTermRef(argFrameP(fr, arg_catcher-1));
+
+      if ( !unify_finished(catcher, reason) )
+	goto out;
+    }
+
+			/* Call the cleanup handler */
+    fr = (LocalFrame)valTermRef(fref);
+    clean = consTermRef(argFrameP(fr, arg_cleanup-1));
+    if ( saveWakeup(&wstate, FALSE) )
+    { int rval;
+
+      startCritical();
+      rval = call_term(contextModule(fr), clean);
+      if ( !endCritical() )
+	rval = FALSE;
+      if ( !rval && exception_term )
+	wstate.flags |= WAKEUP_KEEP_URGENT_EXCEPTION;
+      restoreWakeup(&wstate);
+    }
+
+  out:
     PL_close_foreign_frame(cid);
   }
 }
@@ -1333,6 +1245,8 @@ callBreakHook(DECL_LD LocalFrame frame, Choice bfr,
   wakeup_state wstate;
   size_t pc_offset;
 
+  SAVE_PTRS();
+
   *pop = 0;
   if (op == B_UNIFY_VAR || op == B_UNIFY_FIRSTVAR)
   { LD->slow_unify = TRUE;
@@ -1343,13 +1257,11 @@ callBreakHook(DECL_LD LocalFrame frame, Choice bfr,
   if ( !getProcDefinition(proc)->impl.any.defined )
     goto default_action;
 
-  if ( strchr(codeTable[op].argtype, CA1_FVAR) )
+  if ( strchr(VM_ARGTYPES(&codeTable[op]), CA1_FVAR) )
     pc_offset = stepPC(PC)-PC;
   else
     pc_offset = 0;
 
-  SAVE_PTRS();
-
   /* make enough space to avoid GC/shift in the	critical region*/
   if ( !hasGlobalSpace(10) )
   { int rc;
@@ -1363,19 +1275,21 @@ callBreakHook(DECL_LD LocalFrame frame, Choice bfr,
   if ( saveWakeup(&wstate, FALSE) )
   { if ( (cid=PL_open_foreign_frame()) )
     { term_t argv = PL_new_term_refs(7);
-      Clause clause = frame->clause->value.clause;
+      Clause clause;
       qid_t qid;
 
       RESTORE_PTRS();
+
+      clause = frame->clause->value.clause;
+      PL_put_frame(argv+2, frame); /* first, as it may move */
       PL_put_clref(argv+0, clause);
       PL_put_intptr(argv+1, PC - clause->codes);
-      PL_put_frame(argv+2, frame);
       PL_put_choice(argv+3, bfr);
       PL_put_bool(argv+5, debugstatus.debugging);
       if ( ( op == B_UNIFY_EXIT &&
-             put_call_goal(argv+4, GD->procedures.equals2) &&
-             PL_cons_functor_v(argv+4, FUNCTOR_call1, argv+4) ) ||
-           put_vm_call(argv+4, frameref, PC, op, pc_offset != 0, pop) )
+	     put_call_goal(argv+4, GD->procedures.equals2) &&
+	     PL_cons_functor_v(argv+4, FUNCTOR_call1, argv+4) ) ||
+	   put_vm_call(argv+4, frameref, PC, op, pc_offset != 0, pop) )
       { DEBUG(CHK_SECURE, checkStacks(NULL));
 	if ( (qid = PL_open_query(MODULE_user,
 				  PL_Q_NODEBUG|PL_Q_PASS_EXCEPTION, proc, argv)) )
@@ -1529,15 +1443,29 @@ __do_undo(DECL_LD mark *m)
       setVar(*p);
   }
 
+  DEBUG(CHK_SECURE,
+	{ for(tt = tTop; --tt >= mt;)
+	    tt->address = (Word)0xbfbfbfbf;
+	});
+
   tTop = mt;
+
+  Word ngtop;
   if ( LD->frozen_bar > m->globaltop )
   { DEBUG(CHK_SECURE, assert(gTop >= LD->frozen_bar));
     reclaim_attvars(LD->frozen_bar);
-    gTop = LD->frozen_bar;
+    ngtop = LD->frozen_bar;
   } else
   { reclaim_attvars(m->globaltop);
-    gTop = m->globaltop;
+    ngtop = m->globaltop;
   }
+
+  DEBUG(CHK_SECURE,
+	{ for(Word p = gTop; --p > ngtop;)
+	    *p = 0xbfbfbfbf;
+	});
+
+  gTop = ngtop;
 }
 
 
@@ -2160,10 +2088,12 @@ dbgRedoFrame(DECL_LD LocalFrame fr, choice_type cht)
 
 #endif /*O_DEBUGGER*/
 
-#define exception_hook(pqid, fr, catchfr_ref) LDFUNC(exception_hook, pqid, fr, catchfr_ref)
+#define exception_hook(pqid, fr, catchfr_ref) \
+	LDFUNC(exception_hook, pqid, fr, catchfr_ref)
+
 static int
 exception_hook(DECL_LD qid_t pqid, term_t fr, term_t catchfr_ref)
-{ if ( PROCEDURE_exception_hook4->definition->impl.clauses.first_clause )
+{ if ( PROCEDURE_exception_hook5->definition->impl.clauses.first_clause )
   { if ( !LD->exception.in_hook )
     { wakeup_state wstate;
       qid_t qid;
@@ -2174,7 +2104,7 @@ exception_hook(DECL_LD qid_t pqid, term_t fr, term_t catchfr_ref)
       if ( !saveWakeup(&wstate, TRUE) )
 	return FALSE;
 
-      av = PL_new_term_refs(4);
+      av = PL_new_term_refs(5);
       PL_put_term(av+0, exception_bin);
       PL_put_frame(av+2, (LocalFrame)valTermRef(fr));
 
@@ -2196,10 +2126,11 @@ exception_hook(DECL_LD qid_t pqid, term_t fr, term_t catchfr_ref)
       } else
       { PL_put_frame(av+3, NULL);	/* puts 'none' */
       }
+      PL_put_bool(av+4, debugstatus.debugging);
 
       startCritical();
       qid = PL_open_query(MODULE_user, PL_Q_NODEBUG|PL_Q_CATCH_EXCEPTION,
-			  PROCEDURE_exception_hook4, av);
+			  PROCEDURE_exception_hook5, av);
       rc = PL_next_solution(qid);
       rc = endCritical() && rc;
       debug = debugstatus.debugging;
@@ -2233,7 +2164,7 @@ exception_hook(DECL_LD qid_t pqid, term_t fr, term_t catchfr_ref)
 
       return rc;
     } else
-    { PL_warning("Recursive exception in prolog_exception_hook/4");
+    { PL_warning("Recursive exception in prolog:prolog_exception_hook/5");
     }
   }
 
@@ -2372,7 +2303,7 @@ static void
 discardFrame(DECL_LD LocalFrame fr)
 { Definition def = fr->predicate;
 
-  DEBUG(2, Sdprintf("discard #%d running %s\n",
+  DEBUG(2, Sdprintf("discard #%zd running %s\n",
 		    loffset(fr),
 		    predicateName(fr->predicate)));
 
@@ -2419,7 +2350,7 @@ chp_chars(Choice ch)
 { GET_LD
   static char buf[256];
 
-  Ssnprintf(buf, sizeof(buf), "Choice at #%ld for frame #%ld (%s), type %s",
+  Ssnprintf(buf, sizeof(buf), "Choice at #%zd for frame #%zd (%s), type %s",
 	    loffset(ch), loffset(ch->frame),
 	    predicateName(ch->frame->predicate),
 	    ch->type == CHP_JUMP ? "JUMP" :
@@ -2669,7 +2600,7 @@ PL_open_query(Module ctx, int flags, Procedure proc, term_t args)
   qf->saved_ltop = lTop;
 
 					/* fill top-frame */
-  top	             = &qf->top_frame;
+  top		     = &qf->top_frame;
   top->parent        = NULL;
   top->predicate     = PROCEDURE_dc_call_prolog->definition;
   top->programPointer= NULL;
@@ -2707,6 +2638,7 @@ PL_open_query(Module ctx, int flags, Procedure proc, term_t args)
   qf->foreign_frame	= 0;
   qf->flags		= flags;
   qf->saved_environment = environment_frame;
+  assert(parentFrame(top) == environment_frame);
   qf->saved_bfr		= LD->choicepoints;
   qf->aSave             = aTop;
   qf->solutions         = 0;
@@ -2980,16 +2912,10 @@ typedef enum
   uwrite				/* Unification in write mode */
 } unify_mode;
 
-/* HACK: uncomment the following line to enable function mode */
-/* #define O_VMI_FUNCTIONS 1 */
-/* #define VMI_REGISTER_VARIABLES 1 */
-/* #define VMI_USE_REGISTER_VARIABLES 1 */
 /* registers here MUST be in the list of callee-saved registers! */
 #define LD_REGISTER "rbx"
 #define REGFILE_REGISTER "r12"
 
-#include "pentium.h"
-
 #if O_VMI_FUNCTIONS
 struct register_file;
 #define VMI_RETTYPE Code
@@ -3066,8 +2992,8 @@ typedef struct register_file
  * and just before exit from an instruction (i.e. before goto/return/etc).
  * For profiling/tracing purposes only, and not applied to VMH's.
  */
-#define VMI_ENTER(n)		count(n, PC); START_PROF(n, #n);
-#define VMI_EXIT		END_PROF();
+#define VMI_ENTER(n)		(void)(n);
+#define VMI_EXIT		(void)0;
 
 /* Components of VMI/VMH macro expansion. The underscore-prefix macros
  * get defined per-implementation.
@@ -3106,20 +3032,20 @@ typedef struct register_file
 #define _VMH_EPILOGUE			;
 
 /* Same syntax as VMH_GOTO, but handles profiling as if it were an instruction */
-#define VMH_GOTO_AS_VMI(n,args...) do { VMI_EXIT; \
-					VMI_ENTER(n); \
-					VMH_GOTO(n, args); \
-				      } while(0)
+#define VMH_GOTO_AS_VMI(n,...) do { VMI_EXIT; \
+				    VMI_ENTER(n); \
+				    VMH_GOTO(n, __VA_ARGS__); \
+				  } while(0)
 
 
 /* Helper macros for rendering VMH arguments */
 #define HEAD(h, ...) h
-#define TAIL(_, t...) (t)
-#define VMH_ARGS0(n,at,an,f,asep...)
-#define VMH_ARGS1(n,at,an,f,asep...) f(n, HEAD at, HEAD an)
-#define VMH_ARGS2(n,at,an,f,asep...) f(n, HEAD at, HEAD an) asep VMH_ARGS1(n, TAIL at, TAIL an, f, asep)
-#define VMH_ARGS3(n,at,an,f,asep...) f(n, HEAD at, HEAD an) asep VMH_ARGS2(n, TAIL at, TAIL an, f, asep)
-#define VMH_ARGS4(n,at,an,f,asep...) f(n, HEAD at, HEAD an) asep VMH_ARGS3(n, TAIL at, TAIL an, f, asep)
+#define TAIL(_, ...) (__VA_ARGS__)
+#define VMH_ARGS0(n,at,an,f,...)
+#define VMH_ARGS1(n,at,an,f,...) f(n, HEAD at, HEAD an)
+#define VMH_ARGS2(n,at,an,f,...) f(n, HEAD at, HEAD an) __VA_ARGS__ VMH_ARGS1(n, TAIL at, TAIL an, f, __VA_ARGS__)
+#define VMH_ARGS3(n,at,an,f,...) f(n, HEAD at, HEAD an) __VA_ARGS__ VMH_ARGS2(n, TAIL at, TAIL an, f, __VA_ARGS__)
+#define VMH_ARGS4(n,at,an,f,...) f(n, HEAD at, HEAD an) __VA_ARGS__ VMH_ARGS3(n, TAIL at, TAIL an, f, __VA_ARGS__)
 #define COMMA_TYPE_ARG(n,at,an) , at an
 #define TYPE_ARG_SEMI(n,at,an)	at an ;
 #define VMH_ARGS(n) A_PASTE(VMH_ARGS, VMH_ARGCOUNT(n))(n, (VMH_ARGTYPES(n)), (VMH_ARGNAMES(n)), TYPE_ARG_SEMI)
@@ -3127,8 +3053,18 @@ typedef struct register_file
 /* Define struct types for all the helper argument lists */
 #define VMH_ARGSTRUCT(Name)		struct helper_args_ ## Name
 
+/* GCC and CLang allow for struct name {}, i.e., an empty struct */
+#if O_EMPY_STRUCTS
+#define VMH_PAD_STRUCT
+#define VMH_INIT_ARGSTRUCT(...) {__VA_ARGS__}
+#else
+#undef HAVE_EMPTY_STRUCT
+#define VMH_PAD_STRUCT int _no_empty_struct;
+#define VMH_INIT_ARGSTRUCT(...) {0, ##__VA_ARGS__}
+#endif
+
 FOREACH_VMH(T_EMPTY,
-  ,VMH_ARGSTRUCT, { ,VMH_ARGS, };
+  ,VMH_ARGSTRUCT, {VMH_PAD_STRUCT ,VMH_ARGS, };
 )
 
 #define ASSIGN_ARG(n,at,an)		at an = HELPER_ARGS(n).an;
@@ -3142,7 +3078,7 @@ FOREACH_VMH(T_EMPTY,
 #define _NEXT_INSTRUCTION		return PC
 #define _SOLUTION_RETURN(val)		SOLUTION_RET = (val); longjmp(EXIT_VM_BUF, 1)
 #define _VMI_GOTO(n)			PC--; return instr_ ## n(VMI_ARG_PASS)
-#define _VMH_GOTO(n,...)		VMH_ARGSTRUCT(n) __args = {__VA_ARGS__}; (void)__args; \
+#define _VMH_GOTO(n,...)		VMH_ARGSTRUCT(n) __args = VMH_INIT_ARGSTRUCT(__VA_ARGS__); (void)__args; \
 					return helper_ ## n(VMI_ARG_PASS, __args)
 #undef _VMI_PROLOGUE
 #define _VMI_PROLOGUE(Ident,f,na,a)	PC++;
@@ -3192,7 +3128,8 @@ static vmi_instr jmp_table[] =
 
 #define HELPER_ARGS(n)			helper_args.n
 #define _VMH_DECLARATION(Name,na,at,an)	helper_ ## Name:
-#define _VMH_GOTO(n,args...)		VMH_ARGSTRUCT(n) __args = {args}; \
+#define _VHM_GOTO(n)			goto helper_ ## n;
+#define _VMH_GOTO(n,...)		VMH_ARGSTRUCT(n) __args = VMH_INIT_ARGSTRUCT(__VA_ARGS__); \
 					HELPER_ARGS(n) = __args; \
 					goto helper_ ## n;
 #define _SOLUTION_RETURN		return
